"""Plotting functions of tmd (untested and more complex ploting)"""

import numpy as _np
from tmd import Topology as _tm
import view as _view
import common as _cm
from tmd.analysis import sort_ph


def _sort_ph_radii(ph):
    """
    Sorts barcode according to length of bars.
    """
    ph_sort = [[p[0], p[1], p[2], _np.abs(p[0] - p[1])] for p in ph]
    ph_sort.sort(key=lambda x: x[3])
    return ph_sort


def _sort_ph_initial(ph):
    """
    Sorts barcode according to length of bars.
    """
    ph_sort = [[p[0], p[1], p[2], _np.abs(p[0] - p[1])] for p in ph]
    ph_sort.sort(key=lambda x: x[2])
    return [p[:2] for p in ph_sort]


def _mirror_ph(ph):
    """
    Sorts barcode according to length of bars.
    """
    ph_mirror = [[p[0], p[1], p[0] - p[1]] for p in ph]
    ph_mirror.sort(key=lambda x: x[2])
    return ph_mirror


def tree_all(tree, plane='xy', feature='radial_distances', title='',
             diameter=True, treecol='b', xlims=None, ylims=None, **kwargs):
    '''Subplot with ph, barcode and tree
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    kwargs['output_path'] = kwargs.get('output_path', None)

    fig1, ax1 = _view.tree(tree, new_fig=True, subplot=221, plane='xy',
                           title=title, treecolor=treecol, diameter=diameter)

    feat = getattr(tree, 'get_section_' + feature)()
    segs = tree.get_segments()

    def _seg_2d(seg):
        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]

        return ((horz1, vert1), (horz2, vert2))

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, feature=feature)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))
    fig1, ax2 = ph_diagram(ph, new_fig=False, subplot=222, color=treecol)
    fig1, ax3 = barcode(ph, new_fig=False, subplot=223, color=treecol)
    fig1, ax4 = ph_image(ph, new_fig=False, subplot=224, xlims=xlims, ylims=ylims)
    _cm.plt.tight_layout(True)

    if kwargs['output_path'] is not None:
        fig = _cm.save_plot(fig=ax1, **kwargs)

    return fig1, ax1


def neu_all(neuron, plane='xy', feature='radial_distances', title='',
            diameter=True, treecol='b', xlims=None, ylims=None, neurite_type='basal', **kwargs):
    '''Subplot with ph, barcode
       and tree within spheres
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    kwargs['output_path'] = kwargs.get('output_path', None)

    fig1, ax1 = _view.neuron(neuron, new_fig=True, subplot=221, plane='xy', neurite_type=[neurite_type],
                             title=title, treecolor=treecol, diameter=diameter)

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_ph_neuron(neuron, feature=feature, neurite_type=neurite_type)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))
    fig1, ax2 = ph_diagram(ph, new_fig=False, subplot=222, color=treecol)
    fig1, ax3 = barcode(ph, new_fig=False, subplot=223, color=treecol)
    fig1, ax4 = ph_image(ph, new_fig=False, subplot=224, xlims=xlims, ylims=ylims)
    _cm.plt.tight_layout(True)

    if kwargs['output_path'] is not None:
        fig = _cm.save_plot(fig=ax1, **kwargs)

    return fig1, ax1


def barcode_radii(ph, new_fig=True, subplot=False, linewidth=1.2, diam_max=2.0, **kwargs):
    """
    Generates a 2d figure (barcode) of the persistent homology
    """
    from pylab import cm

    # Initialization of matplotlib figure and axes.
    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    # Hach for colorbar creation
    Z = [[0,0],[0,0]]
    levels = _np.linspace(0.0,diam_max,200)
    CS3 = _view.common.plt.contourf(Z, levels, cmap=cm.jet)
    ph_sort = sort_ph_radii(ph)

    for ip, p in enumerate(ph_sort):
        ax.plot(p[:2], [ip, ip], c=cm.jet(p[2]/diam_max), linewidth=linewidth)

    kwargs['title'] = kwargs.get('title', 'Barcode of p.h.')
    kwargs['xlabel'] = kwargs.get('xlabel', 'Lifetime')

    _view.common.plt.ylim([-1, len(ph_sort)])
    _view.common.plt.colorbar(CS3)

    return _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def barcode_mirror(ph, new_fig=True, subplot=False, color='b', **kwargs):
    """
    Generates a mirrored 2d figure (barcode) of the persistent homology
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    ph_mirror = _mirror_ph(ph)

    for ip, p in enumerate(ph_mirror):
        if p[2] >= 0:
            ax.plot(p[:2], [ip, ip], c=color)
        if p[2] < 0:
            ax.plot(_np.subtract([0,0],p[:2]), [ip, ip], c=color)

    kwargs['title'] = kwargs.get('title', 'Mirror Barcode of p.h.')
    kwargs['xlabel'] = kwargs.get('xlabel', 'Lifetime')
    _view.common.plt.ylim([-len(ph_mirror), len(ph_mirror)])

    return _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def ph_birth_length(ph, new_fig=True, subplot=False, color='b', **kwargs):
    """
    Generates a 2d figure (ph diagram) of the persistent homology
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    ph_sort = sort_ph(ph)

    bounds = _np.max(_np.max(ph))

    for p in ph_sort:

        ax.scatter(p[0], p[2], c=color)

    #_view.common.plt.plot([0, bounds], [0, bounds], c=color)

    kwargs['title'] = kwargs.get('title', 'Birth-Length diagram')
    kwargs['xlabel'] = kwargs.get('xlabel', 'Birth')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Length')

    return _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def ph_on_tree(tree, new_fig=True, subplot=False, plane='xy', alpha=0.05, **kwargs):
    """
    Generates a 3d figure of the tree and adds
    the corresponding spheres that represent
    important events in the persistent homology
    diagram (birth and death of components).
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = _view.tree(tree, new_fig=new_fig, subplot=subplot, plane=plane, **kwargs)

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, dim=plane)
    else:
        raise Exception('Plane value not recognised')

    for p in ph:

        c1 = _view.common.plt.Circle([tree.x[0], tree.y[0],
                                      tree.z[0]], p[0], alpha=alpha)
        c2 = _view.common.plt.Circle([tree.x[0], tree.y[0],
                                      tree.z[0]], p[1], alpha=alpha)

        ax.add_patch(c1) # pylint: disable=no-member
        ax.add_patch(c2) # pylint: disable=no-member

    return _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def barcode_tree(tree, new_fig=True, plane='xy', output_dir=None, **kwargs):
    """
    Generates a 2d figure (barcode) of the persistent homology
    of a tree as it has been computed by
    Topology.get_persistent_homology method.
    """
    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, dim=plane)
    else:
        raise Exception('Plane value not recognised')

    for ip, p in enumerate(ph):

        ph[ip].append(_np.abs(p[0] - p[1]))

    ph.sort(key=lambda x: x[2])

    for ip, p in enumerate(ph):

        fig, ax = _view.tree(tree, new_fig=new_fig, subplot=121, plane=plane)

        c1 = _view.common.plt.Circle([tree.x[0], tree.y[0]], p[0], alpha=0.2)
        c2 = _view.common.plt.Circle([tree.x[0], tree.y[0]], p[1], alpha=0.2)

        ax.add_patch(c1) # pylint: disable=no-member
        ax.add_patch(c2) # pylint: disable=no-member

        fig, ax = _view.common.get_figure(new_fig=False, subplot=122)

        for ip1, p1 in enumerate(ph):
            if ip1 != ip:
                ax.plot(p1[:2], [ip1, ip1], c='b')
            else:
                ax.plot(p1[:2], [ip1, ip1], c='r')

        kwargs['title'] = kwargs.get('title', 'Barcode of p.h.')
        kwargs['xlabel'] = kwargs.get('xlabel', 'Lifetime')
        kwargs['ylabel'] = kwargs.get('ylabel', '')

        _view.common.plt.ylim([-1, len(ph)])

        _view.common.plot_style(fig, ax, **kwargs)

        if output_dir is not None:
            kwargs['output_path'] = output_dir
            kwargs['output_name'] = 'barcode_' + '0'*(2-len(str(len(tree.get_bifurcations()) - ip))) + str(len(tree.get_bifurcations()) - ip)

        _view.common.save_plot(fig, **kwargs)     

        if output_dir is not None:
            kwargs['output_path'] = None
            kwargs['output_name'] = None


def ph_diagram_tree(tree, new_fig=True, plane='xy', output_dir=None, **kwargs):
    """
    Generates a 2d figure (barcode) of the persistent homology
    of a tree as it has been computed by
    Topology.get_persistent_homology method.
    """
    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, dim=plane)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))

    for ip, p in enumerate(ph):

        ph[ip].append(_np.abs(p[0] - p[1]))

    ph.sort(key=lambda x: x[2])

    for ip, p in enumerate(ph):

        fig, ax = _view.tree(tree, new_fig=new_fig, subplot=121, plane=plane)

        c1 = _view.common.plt.Circle([tree.x[0], tree.y[0]], p[0], alpha=0.2)
        c2 = _view.common.plt.Circle([tree.x[0], tree.y[0]], p[1], alpha=0.2)

        ax.add_patch(c1) # pylint: disable=no-member
        ax.add_patch(c2) # pylint: disable=no-member

        fig, ax = _view.common.get_figure(new_fig=False, subplot=122)

        for ip1, p1 in enumerate(ph):
            if ip1 != ip:
                ax.scatter(p1[0], p1[1], c='b')
            else:
                ax.scatter(p1[0], p1[1], c='r', s=50)

        kwargs['title'] = kwargs.get('title', 'P.H. diagram')
        kwargs['xlabel'] = kwargs.get('xlabel', 'Birth')
        kwargs['ylabel'] = kwargs.get('ylabel', 'Death')

        _view.common.plot_style(fig, ax, **kwargs)

        _view.common.plt.plot([0, bounds], [0, bounds])

        if output_dir is not None:
            kwargs['output_path'] = output_dir
            kwargs['output_name'] = 'ph_' + '0'*(2-len(str(len(tree.get_bifurcations()) - ip))) + str(len(tree.get_bifurcations()) - ip) + '.png'

        _view.common.save_plot(fig, **kwargs)     

        if output_dir is not None:
            kwargs['output_path'] = None
            kwargs['output_name'] = None


def tree_instance(tree, new_fig=True, plane='xy', component_num=1, feature='radial_distances', diameter=True, col='r', treecol='b', **kwargs):
    '''Subplot with ph, barcode and tree within spheres
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    if new_fig:
        fig1, ax1 = _view.tree(tree, new_fig=new_fig, subplot=121, plane='xy', title='', treecolor=treecol, diameter=diameter)
    else:
        fig1, ax1 = _view.common.get_figure(new_fig=new_fig, subplot=121)
    feat = getattr(tree, 'get_section_' + feature)()
    segs = tree.get_segments()

    def _seg_2d(seg):
        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]

        return ((horz1, vert1), (horz2, vert2))

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, feature=feature)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))

    if new_fig:
        fig1, ax2 = barcode(ph, new_fig=False, subplot=222, color=treecol)
        fig1, ax3 = ph_diagram(ph, new_fig=False, subplot=224, color=treecol)
    else:
        fig1, ax2 = _view.common.get_figure(new_fig=new_fig, subplot=222)
        fig1, ax3 = _view.common.get_figure(new_fig=new_fig, subplot=224)

    ph = sort_ph(ph)
    select_section = ph[component_num]

    ax2.plot(select_section[:-1], [component_num, component_num], color=col, linewidth=2.)
    ax3.scatter(select_section[0], select_section[1], color=col, s=50.)

    initial = _np.transpose(tree.get_sections())[_np.where(feat == select_section[0])[0]]
    all_way = _np.array(tree.get_way_to_root(initial[0][1]))

    if select_section[1] != -1:
        final = _np.transpose(tree.get_sections())[_np.where(feat == select_section[1])[0]]
        until = _np.array(tree.get_way_to_root(final[0][1]))
    else:
        until = _np.array(tree.get_way_to_root(0))

    between = _np.setxor1d(all_way, until)

    tmp_segs = _np.array(segs)[between]
    toplot_segs = [_seg_2d(seg) for seg in tmp_segs]
    linewidth = [2 * d * 2 for d in _np.array(tree.d)[between]]

    collection = LineCollection(toplot_segs, color=col, linewidth=linewidth, alpha=1.)
    ax1.add_collection(collection)
    _view.common.plt.tight_layout(True)

    return ax1, collection


def customized_cmap():
    '''Returns a custom cmap'''
    import numpy as np
    import matplotlib.pyplot as plt
    import matplotlib.colors as mcolors

    def make_colormap(seq):
        """Return a LinearSegmentedColormap
        seq: a sequence of floats and RGB-tuples. The floats should be increasing
        and in the interval (0,1).
        """
        seq = [(None,) * 3, 0.0] + list(seq) + [1.0, (None,) * 3]
        cdict = {'red': [], 'green': [], 'blue': []}
        for i, item in enumerate(seq):
            if isinstance(item, float):
                r1, g1, b1 = seq[i - 1]
                r2, g2, b2 = seq[i + 1]
                cdict['red'].append([item, r1, r2])
                cdict['green'].append([item, g1, g2])
                cdict['blue'].append([item, b1, b2])
        return mcolors.LinearSegmentedColormap('CustomMap', cdict)

    c = mcolors.ColorConverter().to_rgb

    rvb = make_colormap(
            [(0.9, 0.9, 0.98), 0.01, (0.8, 0.8, 0.98), 0.05,
             (0.7, 0.7, 0.98), 0.10, (0.6, 0.6, 0.98), 0.12,
             (0.5, 0.5, 0.98), 0.15, (0.2, 0.2, 1.0), 0.18,
             (0.0, 0.5, 1.0), 0.20, (0.0, 0.8, 0.8), 0.25,
             (0.0, 0.9, 0.5), 0.33, (0.0, 0.95, 0.2), 0.35,
             (0.4, 0.95, 0.2), 0.4, (0.6, 0.95, 0.1), 0.45,
             (0.98, 0.95, 0.0), 0.5, (0.99, 0.99, 0.0), 0.55,
             (0.99, 0.8, 0.0), 0.60, (0.99, 0.65, 0.0), 0.67,
             (0.99, 0.45, 0.0), 0.7, (0.99, 0.15, 0.0), 0.75,
             (1.0, 0.0, 0.0), 0.8, (0.9, 0.0, 0.0), 0.85,
             (0.75, 0.0, 0.0), 0.999, (0.55, 0.0, 0.0)])

    return rvb


def gaussian_kernel_resample(ph, num_samples=None, scale_persistent_comp=10):
    '''Plots the gaussian kernel of the ph diagram that is given.
    '''
    from scipy import stats
    from numpy.random import multivariate_normal
    import numpy as _np

    values = _np.transpose(ph)
    kernel = stats.gaussian_kde(values)

    if num_samples is None:
        num_samples = len(ph)

    return _np.transpose(kernel.resample(size=num_samples))


def gaussian_kernel_rot(ph, new_fig=True, subplot=111, xlims=None, ylims=None, angle=0, **kwargs):
    '''Plots the gaussian kernel of the ph diagram that is given.
    '''
    from scipy import stats

    def rotation(x, y, angle=0.0):
        return [_np.cos(angle)*x - _np.sin(angle)*y,
                _np.sin(angle)*x + _np.cos(angle)*y]

    ph_r = [rotation(i[0], i[1], angle=angle) for i in ph]

    xmin = min(_np.transpose(ph_r)[0])
    xmax = max(_np.transpose(ph_r)[0])
    ymin = min(_np.transpose(ph_r)[1])
    ymax = max(_np.transpose(ph_r)[1])

    if xlims is None:
        xlims = [xmin, xmax]
    if ylims is None:
        ylims = [ymin, ymax]

    X, Y = _np.mgrid[xlims[0]:xlims[1]:100j, ylims[0]:ylims[1]:100j]

    values = _np.transpose(ph_r)
    kernel = stats.gaussian_kde(values)
    positions = _np.vstack([X.ravel(), Y.ravel()])
    Z = _np.reshape(kernel(positions).T, X.shape)
    Zn = Z / _np.max(Z)

    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    ax.pcolor(Zn, vmin=0., vmax=1., cmap=_view.common.plt.cm.inferno)

    return Z, _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def gaussian_kernel_weighted(ph, new_fig=True, subplot=111, xlims=None, ylims=None, **kwargs):
    '''Plots the gaussian kernel of the ph diagram that is given.
    '''
    from scipy import stats
    xmin = min(_np.transpose(ph)[0])
    xmax = max(_np.transpose(ph)[0])
    ymin = min(_np.transpose(ph)[1])
    ymax = max(_np.transpose(ph)[1])

    if xlims is None:
        xlims = [xmin, xmax]
    if ylims is None:
        ylims = [ymin, ymax]

    X, Y = _np.mgrid[xlims[0]:xlims[1]:100j, ylims[0]:ylims[1]:100j]

    values = _np.transpose(ph)
    kernel = stats.gaussian_kde(values)
    positions = _np.vstack([X.ravel(), Y.ravel()])
    Z = _np.reshape(kernel(positions).T, X.shape)
    Zn = Z / _np.max(Z)

    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    ax.pcolor(Zn, vmin=0., vmax=1., cmap=_view.common.plt.cm.inferno)

    return Z, _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def gaussian_kernel_superposition(ph, new_fig=True, subplot=111, xlims=None, ylims=None, color='r', **kwargs):
    '''Plots the gaussian kernel
       of the ph diagram that is given.
    '''
    from scipy import stats
    xmin = min(_np.transpose(ph)[0])
    xmax = max(_np.transpose(ph)[0])
    ymin = min(_np.transpose(ph)[1])
    ymax = max(_np.transpose(ph)[1])

    if xlims is None:
        xlims = [xmin, xmax]
    if ylims is None:
        ylims = [ymin, ymax]

    X, Y = _np.mgrid[xlims[0]:xlims[1]:100j, ylims[0]:ylims[1]:100j]

    values = _np.transpose(ph)
    kernel = stats.gaussian_kde(values)
    positions = _np.vstack([X.ravel(), Y.ravel()])
    Z = _np.reshape(kernel(positions).T, X.shape)
    Zn = Z / _np.max(Z)
                                                      
    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    ax.pcolor(Zn, vmin=0., vmax=1., cmap=_view.common.plt.cm.inferno)
    #ax.contour(Z, extent=xlims+ylims)

    for p in ph:

        ax.scatter(p[0], p[1], c=color)

    return _view.common.plot_style(fig=fig, ax=ax, xlim=xlims, ylim=ylims, **kwargs)


def tree_br(tree, plane='xy', feature='radial_distances', title='', diameter=True, treecol='b', **kwargs):
    '''Subplot with ph, barcode
       and tree within spheres
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    kwargs['output_path'] = kwargs.get('output_path', None)

    fig1, ax1 = _view.tree(tree, new_fig=True, subplot=121, plane='xy',
                           title=title, treecolor=treecol, diameter=diameter)

    feat = getattr(tree, 'get_section_' + feature)()
    segs = tree.get_segments()

    def _seg_2d(seg):
        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]

        return ((horz1, vert1), (horz2, vert2))

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, feature=feature)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))

    fig1, ax2 = barcode(ph, new_fig=False, subplot=122, color=treecol, ylabel='')

    _view.common.plt.tight_layout(True)

    if kwargs['output_path'] is not None:
        fig = _view.common.save_plot(fig=ax1, **kwargs)

    return fig1, ax1


def tree_gaussian_kernel(tree, plane='xy', feature='radial_distances', title='', diameter=True, treecol='b', xlims=None, ylims=None, **kwargs):
    '''Subplot with ph, barcode
       and tree within spheres
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    kwargs['output_path'] = kwargs.get('output_path', None)

    fig1, ax1 = _view.tree(tree, new_fig=True, subplot=121, plane='xy',
                           title=title, treecolor=treecol, diameter=diameter)

    feat = getattr(tree, 'get_section_' + feature)()
    segs = tree.get_segments()

    def _seg_2d(seg):
        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]

        return ((horz1, vert1), (horz2, vert2))

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, feature=feature)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))

    fig1, ax2 = gaussian_kernel(ph, new_fig=False, subplot=122, xlims=xlims, ylims=ylims)

    _view.common.plt.tight_layout(True)

    if kwargs['output_path'] is not None:
        fig = _view.common.save_plot(fig=ax1, **kwargs)

    return fig1, ax1


def tree_ph(tree, plane='xy', feature='radial_distances', title='', diameter=True, treecol='b', xlims=None, ylims=None, **kwargs):
    '''Subplot with ph, barcode
       and tree within spheres
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    kwargs['output_path'] = kwargs.get('output_path', None)

    fig1, ax1 = _view.tree(tree, new_fig=True, subplot=121, plane='xy',
                           title=title, treecolor=treecol, diameter=diameter)

    feat = getattr(tree, 'get_section_' + feature)()
    segs = tree.get_segments()

    def _seg_2d(seg):
        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]

        return ((horz1, vert1), (horz2, vert2))

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, feature=feature)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))

    fig1, ax2 = ph_diagram(ph, new_fig=False, subplot=122, color=treecol)

    _view.common.plt.tight_layout(True)

    if kwargs['output_path'] is not None:
        fig = _view.common.save_plot(fig=ax1, **kwargs)

    return fig1, ax1


def tree_evol(tree, plane='xy', feature='radial_distances', title='', diameter=True, treecol='b', xlims=None, ylims=None, xlim=None, ylim=None, **kwargs):
    '''Subplot with ph, barcode
       and tree within spheres
    '''
    from tmd import utils as _utils
    from matplotlib.collections import LineCollection

    kwargs['output_path'] = kwargs.get('output_path', None)

    fig1, ax1 = _view.tree(tree, new_fig=True, subplot=121, plane='xy',
                           title=title, treecolor=treecol, diameter=diameter)

    feat = getattr(tree, 'get_section_' + feature)()
    segs = tree.get_segments()

    def _seg_2d(seg):
        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]

        return ((horz1, vert1), (horz2, vert2))

    if plane in ['xy', 'yx', 'zx', 'xz', 'yz', 'zy']:
        ph = _tm.methods.get_persistence_diagram(tree, feature=feature)
    else:
        raise Exception('Plane value not recognised')

    bounds = max(max(ph))

    fig1, ax3 = barcode(ph, new_fig=False, subplot=222, color=treecol, xlim=xlim, ylim=ylim)

    fig1, ax4 = gaussian_kernel(ph, new_fig=False, subplot=224, xlims=xlims, ylims=ylims)

    _view.common.plt.tight_layout(True)

    if kwargs['output_path'] is not None:
        fig = _view.common.save_plot(fig=ax1, **kwargs)

    return fig1, ax1


def image_diff_time(Z_sequence, time_steps=100, new_fig=True, subplot=111, xlims=None, ylims=None, **kwargs):
    """Takes as input a set of images
       as exported from the gaussian kernel
       plotting function, and plots
       their difference.
    """
    

    xmin = min(Z1[1][1].get_xlim() + Z2[1][1].get_xlim())
    xmax = max(Z1[1][1].get_xlim() + Z2[1][1].get_xlim())
    ymin = min(Z1[1][1].get_ylim() + Z2[1][1].get_ylim())
    ymax = max(Z1[1][1].get_ylim() + Z2[1][1].get_ylim())

    if xlims is None:
        xlims = [xmin, xmax]
    if ylims is None:
        ylims = [ymin, ymax]

    X, Y = _np.mgrid[xlims[0]:xlims[1]:100j, ylims[0]:ylims[1]:100j]

    img1 = Z1[0]/Z1[0].max()
    img2 = Z2[0]/Z2[0].max()

    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    _view.common.plt.pcolor(X, Y, img2 - img1, vmin=-1.0, vmax=1.0)

    _view.common.plt.colorbar()

    kwargs['xlim'] = xlims
    kwargs['ylim'] = ylims

    return _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def plot_simple_tree(tr, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0, treecolor='b', alpha=1.0, **kwargs):
    '''Generates a 2d figure of the tree.
    '''
    from matplotlib.collections import LineCollection
    from tmd import utils as _utils

    # Initialization of matplotlib figure and axes.
    fig, ax = _view.common.get_figure(new_fig=new_fig, subplot=subplot)

    # Data needed for the viewer: x,y,z,r
    bounding_box = tr.get_bounding_box()

    def _seg_2d(seg, x_add=0.0, y_add=0.0):

        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz] + x_add
        horz2 = seg[1][horz] + x_add
        vert1 = seg[0][vert] + y_add
        vert2 = seg[1][vert] + y_add

        return ((horz1, vert1), (horz2, vert2))

    segs = [_seg_2d(seg, hadd, vadd) for seg in tr.get_segments()]

    linewidth = [2 * d * 1.0 for d in tr.d]

    #treecolor = 'b'

    # Plot the collection of lines.
    collection = LineCollection(segs, color=treecolor, linewidth=linewidth,
                                alpha=alpha)

    ax.add_collection(collection)

    kwargs['xlim'] = kwargs.get('xlim', [bounding_box[0][_utils.term_dict[plane[0]]] - 20,
                                         bounding_box[1][_utils.term_dict[plane[0]]] + 20])
    kwargs['ylim'] = kwargs.get('ylim', [bounding_box[0][_utils.term_dict[plane[1]]] - 20,
                                         bounding_box[1][_utils.term_dict[plane[1]]] + 20])

    return _view.common.plot_style(fig=fig, ax=ax, **kwargs)


def plot_intermediate(ph_all, colors_bar, tree, colors, counter, linewidth=1., output_path='./'):
    '''plots the tree and the barcode with defined colors'''

    fig, ax1 = _view.common.get_figure(new_fig=True, subplot=122)

    for ipers, pers in enumerate(ph_all):
        ax1.plot(pers[:2], [ipers, ipers], c=colors_bar[ipers], linewidth=linewidth)
        ax1.set_title('Barcode', fontsize=14)

    ax1.set_ylim([-1, len(ph_all)])
    ax1.set_xlim([-10, _np.max(ph_all)])

    ax1.set_xlabel('Distance from soma', fontsize=14)

    ax1.spines['right'].set_visible(False)
    ax1.spines['top'].set_visible(False)
    # ax1.spines['left'].set_visible(False)
    ax1.yaxis.set_ticks_position('left')
    ax1.xaxis.set_ticks_position('bottom')
    ax1.set_yticks([])

    fig, ax2 = plot_simple_tree(tree, new_fig=False, subplot=121, treecolor=colors, diameter=False)

    ax2.spines['right'].set_visible(False)
    ax2.spines['top'].set_visible(False)
    ax2.spines['left'].set_visible(False)
    ax2.yaxis.set_ticks_position('left')
    ax2.xaxis.set_ticks_position('bottom')
    ax2.set_yticks([])

    _view.common.plot_style(fig=fig, ax=ax2, output_path=output_path,
                            title='Morphology', tight=True, xlabel='Length (um)', ylabel='',
                            output_name=str(0)*(3 - len(str(counter))) + str(counter))


def plot_persistent_homology_video(tree, feature='radial_distances', linewidth=1.0, c1='b', c2='w', c3='w', c4='b', c5='r', output_path='./', **kwargs):
    '''Method to extract ph from tree that contains mutlifurcations
    and generate a video of the process'''

    ph_all = _tm.methods.get_persistence_diagram(tree)

    ph_all = _sort_ph_initial(ph_all)

    colors_bar = _np.array([c4 for s in xrange(len(ph_all))])

    ph = []

    rd = getattr(tree, 'get_point_' + feature)(**kwargs)

    colors = _np.array([c1 for s in xrange(len(tree.x) - 1)])

    counter = 5

    plot_intermediate(ph_all, colors_bar, tree, colors, counter, output_path=output_path)

    counter = counter + 1

    active = tree.get_bif_term() == 0

    beg, end = tree.get_sections_2()

    beg = _np.array(beg)
    end = _np.array(end)

    parents = {e: b for b, e in zip(beg, end)}
    children = {b: end[_np.where(beg == b)[0]] for b in _np.unique(beg)}

    colors_bar = _np.array([c3 for s in xrange(len(ph_all))])

    plot_intermediate(ph_all, colors_bar, tree, colors, counter, output_path=output_path)

    counter = counter + 1

    active_paths = {}

    while len(_np.where(active)[0]) > 1:
        alive = list(_np.where(active)[0])

        for l in alive:

            p = parents[l]
            c = children[p]

            if _np.alltrue(active[c]):

                to_modify = []
                to_modify_bars = []

                active[p] = True
                active[c] = False

                mx = _np.argmax(abs(rd[c]))
                mx_id = c[mx]

                alive.remove(mx_id)
                c = _np.delete(c, mx)

                for ci in c:
                    ph.append([rd[ci], rd[p]])
                    #colors_bar[_np.where(_np.array(ph_all) == [rd[ci], rd[p]])[0][1]] = c4
                    to_modify_bars = to_modify_bars + [_np.where(_np.array(ph_all) == [rd[ci], rd[p]])[0][1]]

                    alive.remove(ci)
                    
                    # colors[tree.get_way_to_section_start(ci - 1)] = c2
                    to_modify = to_modify + tree.get_way_to_section_start(ci - 1)

                    if ci < len(colors):
                        # colors[ci] = c2
                        to_modify = to_modify + [ci]

                    key = int(ci)

                    while key in active_paths:

                        # colors[tree.get_way_to_section_start(active_paths[key] - 1)] = c2
                        to_modify = to_modify + tree.get_way_to_section_start(active_paths[key] - 1)
                        if active_paths[key] < len(colors):
                            # colors[active_paths[key]] = c2
                            to_modify = to_modify + [active_paths[key]]
                        key = active_paths.pop(key)

                active_paths[int(p)] = int(mx_id)

                alive.append(p)

                rd[p] = rd[mx_id]

                colors[to_modify] = c5
                colors_bar[to_modify_bars] = c5

                plot_intermediate(ph_all, colors_bar, tree, colors, counter, output_path=output_path)

                counter = counter + 1

                colors[to_modify] = c2
                colors_bar[to_modify_bars] = c4

                plot_intermediate(ph_all, colors_bar, tree, colors, counter, output_path=output_path)

                counter = counter + 1

    to_modify = []

    ph.append([rd[_np.where(active)[0][0]], 0]) # Add the last alive component

    key = int(_np.where(active)[0][0])
    way = tree.get_way_to_section_start(key - 1)
    way.remove(-1)
    # colors[way] = c2
    to_modify = to_modify + way

    if key < len(colors):
        # colors[key] = c2
        to_modify = to_modify + [key]

    while key in active_paths:
        # colors[tree.get_way_to_section_start(active_paths[key] - 1)] = c2
        to_modify = to_modify + tree.get_way_to_section_start(active_paths[key] - 1)
        if active_paths[key] < len(colors):
            # colors[active_paths[key]] = c2
            to_modify = to_modify + [active_paths[key]]
        key = active_paths.pop(key)

    colors_bar[_np.where(_np.array(ph_all) == [rd[_np.where(active)[0][0]], 0])[0][1]] = c5

    colors[to_modify] = c5

    plot_intermediate(ph_all, colors_bar, tree, colors, counter, output_path=output_path)

    counter = counter + 1

    colors[to_modify] = c2

    colors_bar[_np.where(_np.array(ph_all) == [rd[_np.where(active)[0][0]], 0])[0][1]] = c4

    plot_intermediate(ph_all, colors_bar, tree, colors, counter, output_path=output_path)

    # plot_intermediate(ph_all, colors_bar, tree, colors, counter)

    return ph

def multiple_trees_plot(trees, phs, xlim=None, ylim=None, title_1='Asymmetry'):
    fig = plt.figure()
    N = len(trees)
    lims = [np.min(np.min(phs)) - 5 , np.max(np.max(phs)) + 5]

    for i,tr in enumerate(trees):
        view.view.tree(tr, new_fig=False, subplot=(3,N,i+1),
                       title=title_1, plane='xy',
                       diameter=False, linewidth=0.8, treecolor='black')

        view.plot.barcode(phs[i], new_fig=False, subplot=(3,N,i+1+N) , title='', xlim=lims)
        view.plot.ph_image(phs[i], new_fig=False, subplot=(3,N,i+1+2*N) , title='', xlims=lims, ylims=lims)


def polar_plot_custom_color(population, bins=25, apical_color='purple', basal_color='r',
                            edgecolor=None, alpha=0.8):
    '''
    '''
    fig = _cm.plt.figure()
    ax = fig.add_axes([0.1, 0.1, 0.8, 0.8], polar=True)

    input_data1 = _get_polar_data(population, neurite_type='basal', bins=bins)
    input_data2 = _get_polar_data(population, neurite_type='apical', bins=bins)

    maximum = _np.max(_np.array(input_data1)[:,2].tolist() + _np.array(input_data2)[:,2].tolist())

    theta = _np.array(input_data1)[:,0]
    radii = _np.array(input_data1)[:,2] / maximum
    width = 2 * _np.pi / len(input_data1)
    bars = ax.bar(theta, radii, width=width, edgecolor=edgecolor,
                  bottom=0.0, alpha=alpha, color=basal_color)

    theta = _np.array(input_data2)[:,0]
    radii = _np.array(input_data2)[:,2] / maximum
    width = 2 * _np.pi / len(input_data2)
    bars = ax.bar(theta, radii, width=width, edgecolor=edgecolor,
                  bottom=0.0, alpha=alpha, color=apical_color)



'''
tmd transformation algorithms
'''
import numpy as np


def tmd_scale(barcode, thickness):
    '''Only the first two components will be scaled according to
       the thickness parameter, because they correspond to
       spatial coordinates
    '''
    scaling_factor = np.ones(len(barcode[0]), dtype=np.float)
    scaling_factor[:2] = thickness
    return np.multiply(barcode, scaling_factor).tolist()



'''
tmd Soma's methods
'''
import numpy as np


def get_center(self):
    """
    Soma method to get the center of the soma.
    """
    x_center = np.mean(self.x)
    y_center = np.mean(self.y)
    z_center = np.mean(self.z)

    return np.array([x_center, y_center, z_center])


def get_diameter(self):
    """
    Soma method to get the diameter of the soma.
    """
    if len(self.x) == 1:
        diameter = self.d[0]
    else:
        center = self.get_center()
        diameter = np.mean(np.sqrt(np.power(self.x - center[0], 2) +
                                   np.power(self.y - center[1], 2) +
                                   np.power(self.z - center[2], 2)))
    return diameter



'''Test tmd.Tree'''
from nose import tools as nt
import numpy as np
from tmd.Tree import Tree

x1 = np.array([0.,  3.,  4.,  5.,  5.])
y1 = np.array([0.,  4.,  5.,  6.,  6.])
z1 = np.array([ 0.,  5.,  6.,  7.,  7.])
d1 = np.array([12.,  12.,  14.,  16.,  16.])
t1 = np.array([1, 1, 1, 1, 1])
p1 = np.array([-1,  0,  1, 2, 3])

x2 = np.array([0.,  3.,  4.,  5.,  4.])

def test_tree_init_():
    tree1 = Tree.Tree(x=x1, y=y1, z=z1, d=d1, t=t1, p=p1)
    nt.ok_(np.allclose(tree1.x, x1))
    nt.ok_(np.allclose(tree1.y, y1))
    nt.ok_(np.allclose(tree1.z, z1))
    nt.ok_(np.allclose(tree1.d, d1))
    nt.ok_(np.allclose(tree1.t, t1))
    nt.ok_(np.allclose(tree1.p, p1))

def test_tree_is_equal():
    tree1 = Tree.Tree(x=x1, y=y1, z=z1, d=d1, t=t1, p=p1)
    tree2 = Tree.Tree(x=x1, y=y1, z=z1, d=d1, t=t1, p=p1)
    nt.ok_(tree1.is_equal(tree2))
    tree3 = Tree.Tree(x=x2, y=y1, z=z1, d=d1, t=t1, p=p1)
    nt.ok_(not tree1.is_equal(tree3))

def test_copy_tree():
    tree1 = Tree.Tree(x=x1, y=y1, z=z1, d=d1, t=t1, p=p1)
    tree2 = tree1.copy_tree()
    nt.ok_(tree1.is_equal(tree2))
    nt.ok_(tree1 != tree2)



# Example to extract the persistence diagram from a neuronal tree

# Step 1: Import the tmd module
import tmd

# Step 2: Load your morphology
filename = '../tests/data/valid/C010398B-P2.CNG.swc'
neu = tmd.io.load_neuron(filename)

# Step 3: Extract the ph diagram of a tree
tree = neu.neurites[0]
ph = tmd.methods.get_persistence_diagram(tree)

# Step 4: Extract the ph diagram of a neuron's trees
ph_neu = tmd.methods.get_ph_neuron(neu)

# Step 5: Extract the ph diagram of a neuron's trees,
# depending on the neurite_type
ph_apical = tmd.methods.get_ph_neuron(neu, neurite_type='apical')
ph_axon = tmd.methods.get_ph_neuron(neu, neurite_type='axon')
ph_basal = tmd.methods.get_ph_neuron(neu, neurite_type='basal')

# Step 6: Plot the extracted topological data with three different ways
from tmd.view import view, plot

# Visualize the neuron
view.neuron(neu)

# Visualize a selected neurite type or multiple of them
view.neuron(neu, neurite_type=['apical'])

# Visualize the persistence diagram
plot.diagram(ph_apical)

# Visualize the persistence barcode
plot.barcode(ph_apical)

# Visualize the persistence image
plot.persistence_image(ph_apical)



'''
tmd class : Population
'''
from tmd.Neuron import Neuron


class Population(object):
    """
    A Population object is a container for Neurons.
    """

    def __init__(self, name='Pop'):
        '''Creates an empty Population object.
        '''
        self.neurons = []
        self.name = name

    @property
    def axon(self):
        '''Get axon'''
        return [a for n in self.neurons for a in n.axon]

    @property
    def apical(self):
        '''Get apical'''
        return [a for n in self.neurons for a in n.apical]

    @property
    def basal(self):
        '''Get basal'''
        return [a for n in self.neurons for a in n.basal]

    @property
    def undefined(self):
        '''I dont know'''
        return [a for n in self.neurons for a in n.undefined]

    @property
    def neurites(self):
        '''Get neurites'''
        return self.apical + self.axon + self.basal + self.undefined

    @property
    def dendrites(self):
        '''Get dendrites'''
        return self.apical + self.basal

    def append_neuron(self, new_neuron):
        """
        If type of object is neuron it adds
        the new_neuron to the list of neurons
        of the population.
        """
        if isinstance(new_neuron, Neuron.Neuron):
            self.neurons.append(new_neuron)



"""
Visualize neurons and plots persistence barcode, diagrams, images.
Matplotlib required.
"""


try:
    import matplotlib
except ImportError:
    raise ImportError('tmd[viewer] is not installed. ' +
                      'Please install it by doing: pip install tmd[viewer]')

from tmd.view import plot, view



'''Test tmd.soma'''
from nose import tools as nt
import numpy as np
from tmd.Soma import Soma

x1 = np.array([0.,  3.,  4.,  5.,  5.])
y1 = np.array([0.,  4.,  5.,  6.,  6.])
z1 = np.array([ 0.,  5.,  6.,  7.,  7.])
d1 = np.array([12.,  12.,  14.,  16.,  16.])

x2 = np.array([0.,  3.,  4.,  5.,  4.])

def test_soma_init_():
    soma1 = Soma.Soma(x=x1, y=y1, z=z1, d=d1)
    nt.ok_(np.allclose(soma1.x, x1))
    nt.ok_(np.allclose(soma1.y, y1))
    nt.ok_(np.allclose(soma1.z, z1))
    nt.ok_(np.allclose(soma1.d, d1))

def test_soma_is_equal():
    soma1 = Soma.Soma(x=x1, y=y1, z=z1, d=d1)
    soma2 = Soma.Soma(x=x1, y=y1, z=z1, d=d1)
    nt.ok_(soma1.is_equal(soma2))
    soma2 = Soma.Soma(x=x2, y=y1, z=z1, d=d1)
    nt.ok_(not soma1.is_equal(soma2))

def test_copy_soma():
    soma1 = Soma.Soma(x=x1, y=y1, z=z1, d=d1)
    soma2 = soma1.copy_soma()
    nt.ok_(soma1.is_equal(soma2))
    nt.ok_(soma1 != soma2)



'''Test tmd.Neuron'''
from nose import tools as nt
import numpy as np
from tmd.Neuron import Neuron
from tmd.Soma import Soma
from tmd.Tree import Tree
from tmd.utils import tree_type as td

soma_test = Soma.Soma([0.],[0.],[0.],[12.])
soma_test1 = Soma.Soma([0.],[0.],[0.],[6.])
apical_test = Tree.Tree(x=np.array([5., 5.]), y=np.array([6., 6.]), z=np.array([ 7., 7.]),
                       d=np.array([16., 16.]), t=np.array([4, 4]), p=np.array([-1,  0]))
basal_test = Tree.Tree(x=np.array([4.]), y=np.array([5.]), z=np.array([6.]),
                       d=np.array([14.]), t=np.array([3]), p=np.array([-1]))
axon_test = Tree.Tree(x=np.array([ 3.]), y=np.array([4.]), z=np.array([5.]),
                      d=np.array([12.]), t=np.array([2 ]), p=np.array([-1]))

neu_test = Neuron.Neuron()
neu_test.set_soma(soma_test)
neu_test.append_tree(apical_test, td)

def test_neuron_init_():
    neu1 = Neuron.Neuron()
    nt.ok_(neu1.name == 'Neuron')
    nt.ok_(isinstance(neu1.soma, Soma.Soma))
    nt.ok_(neu1.axon == [])
    nt.ok_(neu1.basal == [])
    nt.ok_(neu1.apical == [])
    nt.ok_(neu1.neurites == [])
    neu1 = Neuron.Neuron(name='test')
    nt.ok_(neu1.name == 'test')

def test_neuron_rename():
    neu1 = Neuron.Neuron()
    neu1.rename('test')
    nt.ok_(neu1.name == 'test')

def test_copy_neuron():
    neu1 = Neuron.Neuron()
    neu2 = neu1.copy_neuron()
    nt.ok_(neu1.is_equal(neu2))
    nt.ok_(neu1 != neu2)

def test_neuron_is_equal():
    neu1 = Neuron.Neuron()
    neu1.set_soma(soma_test)
    neu1.append_tree(apical_test, td)
    nt.ok_(neu1.is_equal(neu_test))
    neu1 = Neuron.Neuron()
    neu1.set_soma(soma_test1)
    neu1.append_tree(apical_test, td)
    nt.ok_(not neu1.is_equal(neu_test))
    neu1 = Neuron.Neuron()
    neu1.set_soma(soma_test)
    neu1.append_tree(basal_test, td)
    nt.ok_(not neu1.is_equal(neu_test))

def test_neuron_is_same():
    neu1 = Neuron.Neuron()
    neu1.set_soma(soma_test)
    neu1.append_tree(apical_test, td)
    nt.ok_(neu1.is_same(neu_test))
    neu1.name = 'test_not_same'
    nt.ok_(not neu1.is_same(neu_test))

def test_neuron_set_soma():
    neu1 = Neuron.Neuron()
    neu1.set_soma(soma_test)
    nt.ok_(neu1.soma.is_equal(soma_test))

def test_append_tree():
    neu1 = Neuron.Neuron()
    neu1.append_tree(apical_test, td)
    nt.ok_(len(neu1.neurites) == 1)
    neu1.append_tree(basal_test, td)
    neu1.append_tree(axon_test, td)
    nt.ok_(len(neu1.neurites) == 3)
    nt.ok_(len(neu1.basal) == 1)
    nt.ok_(len(neu1.axon) == 1)
    nt.ok_(len(neu1.apical) == 1)



'''
Contains all the commonly used functions and data
useful for multiple tmd modules.
'''

term_dict = {'x': 0,
             'y': 1,
             'z': 2}

tree_type = {1: 'soma',
             2: 'axon',
             3: 'basal',
             4: 'apical'}



'''Test tmd.soma'''
from nose import tools as nt
import numpy as np
from tmd.Soma import Soma

soma = Soma.Soma([0.],[0.],[0.],[12.])
soma1 = Soma.Soma([0.,0.,0.],[0.,1.,0.],[0.,0.,1.],[0.,0.,0.])

def test_get_center():
    nt.ok_(np.allclose(soma.get_center(), [0., 0., 0.]))
    nt.ok_(np.allclose(soma1.get_center(), [0., 1./3., 1./3.]))

def test_get_diameter():
    nt.ok_(soma.get_diameter() == 12.)
    nt.ok_(soma1.get_diameter() == 0.65403883526363049)



'''Test tmd.io'''
import os
from nose import tools as nt
from numpy.testing import assert_array_equal
from tmd.io import io
import numpy as np
from tmd.Soma import Soma
from tmd.Tree import Tree
import glob

_path = os.path.dirname(os.path.abspath(__file__))
DATA_PATH = os.path.join(_path, 'data')
POP_PATH = os.path.join(DATA_PATH, 'valid')

# Filenames for testing
basic_file = os.path.join(DATA_PATH, 'basic.swc')
nosecids_file = os.path.join(DATA_PATH, 'basic_no_sec_ids.swc')
sample_file = os.path.join(DATA_PATH, 'sample.swc')

sample_h5_v1_file = os.path.join(DATA_PATH, 'sample_v1.h5')
sample_h5_v2_file = os.path.join(DATA_PATH, 'sample_v2.h5')
sample_h5_v0_file = os.path.join(DATA_PATH, 'sample_v0.h5')

neuron_v1 = io.load_neuron(sample_h5_v1_file)
neuron_v2 = io.load_neuron(sample_h5_v2_file)
neuron1 = io.load_neuron(sample_file)

soma_test = Soma.Soma([0.], [0.], [0.], [12.])
soma_test1 = Soma.Soma([0.], [0.], [0.], [6.])
apical_test = Tree.Tree(x=np.array([5., 5.]), y=np.array([6., 6.]), z=np.array([7., 7.]),
                        d=np.array([16., 16.]), t=np.array([4, 4]), p=np.array([-1,  0]))
basal_test = Tree.Tree(x=np.array([4.]), y=np.array([5.]), z=np.array([6.]),
                       d=np.array([14.]), t=np.array([3]), p=np.array([-1]))
axon_test = Tree.Tree(x=np.array([3.]), y=np.array([4.]), z=np.array([5.]),
                      d=np.array([12.]), t=np.array([2]), p=np.array([-1]))


def test_type_dict():
    nt.ok_(io.TYPE_DCT == {'soma': 1,
                           'basal': 3,
                           'apical': 4,
                           'axon': 2})


def test_load_swc_neuron():
    neuron = io.load_neuron(basic_file)
    nt.ok_(neuron.soma.is_equal(soma_test))
    nt.ok_(len(neuron.apical) == 1)
    nt.ok_(len(neuron.basal) == 1)
    nt.ok_(len(neuron.axon) == 1)
    nt.ok_(neuron.apical[0].is_equal(apical_test))
    nt.ok_(neuron.basal[0].is_equal(basal_test))
    nt.ok_(neuron.axon[0].is_equal(axon_test))
    neuron1 = io.load_neuron(sample_file)
    nt.ok_(neuron1.soma.is_equal(soma_test1))
    nt.ok_(len(neuron1.apical) == 0)
    nt.ok_(len(neuron1.basal) == 1)
    nt.ok_(len(neuron1.axon) == 1)
    tree_1 = neuron1.axon[0]
    nt.ok_(np.allclose(tree_1.get_bifurcations(), np.array([10, 20])))
    nt.ok_(np.allclose(tree_1.get_terminations(), np.array([15, 25, 30])))
    try:
        neuron = io.load_neuron(nosecids_file)
        nt.ok_(False)
    except:
        nt.ok_(True)


def test_load_h5_neuron():
    nt.ok_(neuron_v1.soma.is_equal(neuron_v2.soma))
    nt.ok_(neuron_v1.basal[0].is_equal(neuron_v2.basal[0]))
    nt.ok_(neuron_v1.axon[0].is_equal(neuron_v2.axon[0]))
    try:
        neuron = io.load_neuron(sample_h5_v0_file)
        nt.ok_(False)
    except:
        nt.ok_(True)


def test_io_load():
    #neuron_v1 = io.load_neuron(sample_h5_v1_file)
    #neuron_v2 = io.load_neuron(sample_h5_v2_file)
    nt.ok_(neuron1.is_equal(neuron_v1))
    nt.ok_(neuron1.is_equal(neuron_v2))


def test_load_population():
    population = io.load_population(POP_PATH)
    nt.ok_(len(population.neurons) == 5)
    names = np.array([os.path.basename(n.name) for n in population.neurons])

    L = glob.glob(POP_PATH + '/*')
    population1 = io.load_population(L)
    nt.ok_(len(population.neurons) == 5)
    names1 = np.array([os.path.basename(n.name) for n in population1.neurons])
    assert_array_equal(names, names1)
    nt.ok_(population.neurons[0].is_equal(population1.neurons[0]))


def test_tree_type():
    tree_types = {5: 'soma',
                  6: 'axon',
                  7: 'basal',
                  8: 'apical'}

    neuron = io.load_neuron(os.path.join(DATA_PATH, 'basic_exotic_section_types.swc'),
                            soma_type=5,
                            tree_types=tree_types)

    def point(section):
        return np.column_stack([section.x, section.y, section.z])
    assert_array_equal(point(neuron.apical[0]), [[3, 4, 5]])
    assert_array_equal(point(neuron.basal[0]), [[4, 5, 6]])
    assert_array_equal(point(neuron.axon[0]), [[5, 6, 7], [5, 6, 7]])



'''
tmd Topology algorithms implementation
'''
import numpy as np
import scipy.spatial as sp
from tmd.Topology.analysis import sort_ph


def write_ph(ph, output_file='test.txt'):
    '''Writes a persistence diagram in
       an output file.
    '''
    wfile = open(output_file, 'w')

    for p in ph:
        wfile.write(str(p[0]) + ' ' + str(p[1]) + '\n')

    wfile.close()


def get_persistence_diagram(tree, feature='radial_distances', **kwargs):
    '''Method to extract ph from tree that contains mutlifurcations'''
    ph = []

    rd = getattr(tree, 'get_point_' + feature)(**kwargs)

    active = tree.get_bif_term() == 0

    beg, end = tree.get_sections_2()

    beg = np.array(beg)
    end = np.array(end)

    parents = {e: b for b, e in zip(beg, end)}

    if 0 in beg:
        parents[0] = tree.p[0]

    children = {b: end[np.where(beg == b)[0]] for b in np.unique(beg)}

    while len(np.where(active)[0]) > 1:
        alive = list(np.where(active)[0])
        for l in alive:

            p = parents[l]
            c = children[p]

            if np.alltrue(active[c]):
                active[p] = True
                active[c] = False

                mx = np.argmax(abs(rd[c]))
                mx_id = c[mx]

                c = np.delete(c, mx)

                for ci in c:
                    ph.append([rd[ci], rd[p]])

                rd[p] = rd[mx_id]

    ph.append([rd[np.where(active)[0][0]], 0])  # Add the last alive component

    return ph


def _phi_theta(u, v):
    """Computes the angles between vectors u, v
    in the plane x-y (phi angle) and the plane x-z (theta angle).
    Returns phi, theta
    """
    phi1 = np.arctan2(u[1], u[0])
    # pylint: disable=assignment-from-no-return
    theta1 = np.arccos(u[2] / np.linalg.norm(u))

    # pylint: disable=assignment-from-no-return
    phi2 = np.arctan2(v[1], v[0])
    theta2 = np.arccos(v[2] / np.linalg.norm(v))

    delta_phi = phi2 - phi1  # np.abs(phi1 - phi2)
    delta_theta = theta2 - theta1  # np.abs(theta1 - theta2)

    return delta_phi, delta_theta  # dphi, dtheta


def _angles_tree(tree, parID, parEND, ch1ID, ch2ID):
    '''Computes the x-y and x-z angles between parent
       and children within the given tree.
    '''

    dirP = tree.get_direction_between(start_id=parID, end_id=parEND)
    dirU = tree.get_direction_between(start_id=parEND, end_id=ch1ID)
    dirV = tree.get_direction_between(start_id=parEND, end_id=ch2ID)

    phi1, theta1 = _phi_theta(dirP, dirU)
    phi2, theta2 = _phi_theta(dirP, dirV)

    if np.abs(phi1) < np.abs(phi2):
        dphi = phi1
        dtheta = theta1
        delta_phi, delta_theta = _phi_theta(dirU, dirV)
    else:
        dphi = phi2
        dtheta = theta2
        delta_phi, delta_theta = _phi_theta(dirV, dirU)

    return [dphi, dtheta, delta_phi, delta_theta]


def get_angles(tree, beg, parents, children):
    """Returns the angles between all the triplets (parent, child1, child2)
    of the tree"""
    angles = [[0, 0, 0, 0], ]  # Null angle for non bif point

    for b in beg[1:]:

        angleBetween = _angles_tree(tree, parID=parents[b], parEND=b,
                                    ch1ID=children[b][0],
                                    ch2ID=children[b][1])
        angles.append(angleBetween)

    return angles


def get_ph_angles(tree, feature='radial_distances', **kwargs):
    '''Method to extract ph from tree that contains mutlifurcations'''
    ph = []

    rd = getattr(tree, 'get_point_' + feature)(**kwargs)

    active = tree.get_bif_term() == 0

    beg, end = tree.get_sections_2()

    beg = np.array(beg)
    end = np.array(end)

    parents = {e: b for b, e in zip(beg, end)}

    if 0 in beg:
        parents[0] = tree.p[0]

    children = {b: end[np.where(beg == b)[0]] for b in np.unique(beg)}

    angles = get_angles(tree, beg, parents, children)

    while len(np.where(active)[0]) > 1:
        alive = list(np.where(active)[0])
        for l in alive:

            p = parents[l]
            c = children[p]

            if np.alltrue(active[c]):
                active[p] = True
                active[c] = False

                mx = np.argmax(abs(rd[c]))
                mx_id = c[mx]

                c = np.delete(c, mx)

                for ci in c:
                    angID = np.array(angles)[np.where(beg == p)[0][0]]
                    ph.append([rd[ci], rd[p], angID[0], angID[1], angID[2], angID[3]])

                rd[p] = rd[mx_id]

    ph.append([rd[np.where(active)[0][0]], 0, np.nan, np.nan, np.nan, np.nan])

    return ph


def get_ph_radii(tree, feature='radial_distances', **kwargs):
    """Returns the ph diagram enhanced with the corresponding encoded radii"""
    def get_section_mean_radii(tree, beg, end):
        """Returns the mean radii of a section"""
        return [np.mean(tree.d[beg[i]:end[i]]) for i in range(len(beg))]

    ph = []

    rd = getattr(tree, 'get_point_' + feature)(**kwargs)

    active = tree.get_bif_term() == 0

    beg, end = tree.get_sections_2()

    beg = np.array(beg)
    end = np.array(end)

    parents = {e: b for b, e in zip(beg, end)}
    children = {b: end[np.where(beg == b)[0]] for b in np.unique(beg)}

    radii = get_section_mean_radii(tree, beg, end)

    while len(np.where(active)[0]) > 1:
        alive = list(np.where(active)[0])
        for l in alive:

            p = parents[l]
            c = children[p]

            if np.alltrue(active[c]):
                active[p] = True
                active[c] = False

                mx = np.argmax(abs(rd[c]))
                mx_id = c[mx]

                c = np.delete(c, mx)

                for ci in c:
                    radiiID = np.array(radii)[np.where(beg == p)[0][0]]
                    ph.append([rd[ci], rd[p], radiiID])

                rd[p] = rd[mx_id]

    ph.append([rd[np.where(active)[0][0]], 0, radii[beg[0]]])  # Add the last alive component

    return ph


def get_ph_neuron(neuron, feature='radial_distances', neurite_type='all', **kwargs):
    '''Method to extract ph from a neuron that contains mutlifurcations'''

    ph_all = []

    if neurite_type == 'all':
        neurite_list = ['neurites']
    else:
        neurite_list = [neurite_type]

    for t in neurite_list:
        for tr in getattr(neuron, t):
            ph_all = ph_all + get_persistence_diagram(tr, feature=feature, **kwargs)

    return ph_all


def extract_ph(tree, feature='radial_distances', output_file='test.txt',
               sort=False, **kwargs):
    '''Extracts persistent homology from tree'''
    ph = get_persistence_diagram(tree, feature=feature, **kwargs)

    if sort:
        p = sort_ph(ph)
    else:
        p = ph

    write_ph(p, output_file)


def extract_ph_neuron(neuron, feature='radial_distances', output_file=None,
                      neurite_type='all', sort=False, **kwargs):
    '''Extracts persistent homology from tree'''
    ph = get_ph_neuron(neuron, feature=feature, neurite_type='all', **kwargs)

    if sort:
        sort_ph(ph)
    else:
        p = ph

    if output_file is None:
        output_file = 'PH_' + neuron.name + '_' + neurite_type + '.txt'

    write_ph(p, output_file)


def get_lifetime(tree, feature='point_radial_distances'):
    '''Returns the sequence of birth - death times for each section.
    This can be used as the first step for the approximation of P.H.
    of the radial distances of the neuronal branches.
    '''
    begs, ends = tree.get_sections_2()
    rd = getattr(tree, 'get_' + feature)()
    lifetime = np.array(len(begs) * [np.zeros(2)])

    for i, (beg, end) in enumerate(zip(begs, ends)):
        lifetime[i] = np.array([rd[beg], rd[end]])

    return lifetime


def extract_connectivity_from_points(tree, threshold=1.0):
    '''Extract connectivity from list of points'''
    coords = np.transpose([tree.x, tree.y, tree.z])
    distances_matrix = sp.distance.cdist(coords, coords)
    mat = distances_matrix < threshold

    return mat



list_of_modules = ['discriminant_analysis', 'discriminant_analysis', 'tree']

list_of_classifiers = ['LinearDiscriminantAnalysis',
                       'QuadraticDiscriminantAnalysis', 'DecisionTreeClassifier']


def train(mod, classifier, data, labels, **kwargs):
    '''Trains the classifier from mod of sklearn
       with data and targets.
       Returns a fited classifier.
    '''
    import importlib

    clas_mod = importlib.import_module('sklearn.' + mod)
    clf = getattr(clas_mod, classifier)()
    clf.set_params(**kwargs)

    clf.fit(data, labels)

    return clf


def predict(clf, data):
    '''Predict label for data for the trained classifier clf.
       Returns the index of the predicted class
       for each datapoint in data.
    '''
    predict_label = clf.predict([data])

    return predict_label[0]


def leave_one_out(mod, classifier, data, labels, **kwargs):
    '''Leaves one individual out, trains classifier
       with the rest of the data and returns the score
       of matching ids between proposed and predicted labels.
       Score defines how many trials were successful
       as a percentage over the total number of trials.
    '''
    sample_size = len(labels)
    scores = np.zeros(sample_size)

    for ed in range(sample_size):

        # print 'Testing ' + str(ed) + ' ...'

        train_data = data[np.delete(range(sample_size), ed)]
        train_labels = labels[np.delete(range(sample_size), ed)]

        clf = train(mod, classifier, train_data, train_labels, **kwargs)
        predict_label = predict(clf, data[ed])

        # print 'The individual ' + str(ed) + ' is of type ' + str(predict_label)

        scores[ed] = predict_label == labels[ed]

    return np.float(np.count_nonzero(scores)) / sample_size


def leave_one_out_statistics(mod, classifier, data, labels, N=10, **kwargs):
    '''Leaves one individual out, trains classifier
       with the rest of the data and returns the score
       of matching ids between proposed and predicted labels.
       Score defines how many trials were successful
       as a percentage over the total number of trials.
    '''
    sample_size = len(labels)
    scores = np.zeros(sample_size)

    for ed in range(sample_size):

        # print 'Testing ' + str(ed) + ' ...'

        train_data = data[np.delete(range(sample_size), ed)]
        train_labels = labels[np.delete(range(sample_size), ed)]

        clf = train(mod, classifier, train_data, train_labels, **kwargs)

        all_results = []
        for i in range(N):
            all_results.append(predict(clf, data[ed]))

        predict_label = predict(clf, data[ed])
        print 'The individual ', str(ed), ' is of type ', all_results

        scores[ed] = predict_label == labels[ed]

    return np.float(np.count_nonzero(scores)) / sample_size


def leave_perc_out(mod, classifier, data, labels, iterations=10, percent=10, **kwargs):
    '''Leaves one individual out, trains classifier
       with the rest of the data and returns the score
       of matching ids between proposed and predicted labels.
       Score defines how many trials were successful
       as a percentage over the total number of trials.
       Iteration defines the number of trials.
       Percent defines the percentage of the data that will
       define the test set of the classifier.
    '''
    import random

    sample_size = len(labels)
    test_size = int(sample_size * percent / 100.)
    scores = np.zeros(iterations)

    # print sample_size, test_size

    for i in range(iterations):

        random_inds = random.sample(range(0, sample_size), test_size)
        kept = np.delete(range(sample_size), random_inds)

        clf = train(mod, classifier, data[kept], labels[kept], **kwargs)

        sc = 0.0
        for ed in random_inds:
            predict_label = predict(clf, data[ed])
            sc = sc + float(predict_label == labels[ed])

        scores[i] = float(sc) / float(test_size)

    # print len(random_inds), len(kept), len(random_inds) + len(kept)

    return scores  # np.mean(np.count_nonzero(scores))/sample_size


def leave_one_out_mixing(mod, classifier, data, labels, **kwargs):
    '''Leaves one individual out, trains classifier
       with the rest of the data and returns the score
       of matching ids between proposed and predicted labels.
       Score defines how many trials were successful
       as a percentage over the total number of trials.
    '''
    sample_size = len(labels)
    scores = np.zeros(sample_size)

    separation = np.zeros([len(np.unique(labels)), len(np.unique(labels))])

    sizes = np.zeros(len(np.unique(labels)))

    for i in np.unique(labels):
        sizes[int(i - 1)] = len(np.where(labels == i)[0])

    for ed in range(sample_size):

        # print 'Testing ' + str(ed) + ' ...'

        train_data = data[np.delete(range(sample_size), ed)]
        train_labels = labels[np.delete(range(sample_size), ed)]

        clf = train(mod, classifier, train_data, train_labels, **kwargs)
        predict_label = predict(clf, data[ed])

        # print predict_label, labels[ed]
        separation[int(labels[ed] - 1)][int(predict_label - 1)] = separation[int(labels[ed] - 1)
                                                                             ][int(predict_label - 1)] + 1. / sizes[int(labels[ed] - 1)]

        # print 'The individual ' + str(ed) + ' is of type ' + str(predict_label)

        scores[ed] = predict_label == labels[ed]

    return np.float(np.count_nonzero(scores)) / sample_size, separation


def leave_one_out_multiple(mod, classifier, data, labels, n=10, **kwargs):
    '''Leaves one individual out, trains classifier
       with the rest of the data and returns the score
       of matching ids between proposed and predicted labels.
       Score defines how many trials were successful
       as a percentage over the total number of trials.
    '''
    sample_size = len(labels)
    scores = np.zeros(sample_size)

    for ed in range(sample_size):

        # print 'Testing ' + str(ed) + ' ...'

        print 'The individual ' + str(ed) + ' is of type ',

        for ni in range(n):

            train_data = data[np.delete(range(sample_size), ed)]
            train_labels = labels[np.delete(range(sample_size), ed)]

            clf = train(mod, classifier, train_data, train_labels, **kwargs)

            predict_label = predict(clf, data[ed])

            print str(predict_label),

        print ' !'

        scores[ed] = predict_label == labels[ed]

    return np.float(np.count_nonzero(scores)) / sample_size


def multi(dat, tar, m='tree', cl='DecisionTreeClassifier', n=10, randomize=False):
    score = np.zeros(n)
    if not randomize:
        for i in range(n):
            score[i] = leave_one_out(m, cl, dat, tar)
    else:
        for i in range(n):
            score[i] = leave_one_out(m, cl, dat, np.random.randint(
                min(tar), max(tar) + 1, size=len(tar)))

    return mean(score), std(score)



'''
tmd class : Tree
'''
import copy
import numpy as np
import scipy.sparse as sp


class Tree(object):
    '''Tree class'''
    # pylint: disable=import-outside-toplevel
    from tmd.Tree.methods import get_sections_2
    from tmd.Tree.methods import get_sections_only_points
    from tmd.Tree.methods import get_segments
    from tmd.Tree.methods import get_bounding_box
    from tmd.Tree.methods import get_pca
    from tmd.Tree.methods import get_type
    from tmd.Tree.methods import get_direction_between
    from tmd.Tree.methods import get_point_radial_distances
    from tmd.Tree.methods import get_point_radial_distances_time
    from tmd.Tree.methods import get_point_weighted_radial_distances
    from tmd.Tree.methods import get_point_path_distances
    from tmd.Tree.methods import get_point_projection
    from tmd.Tree.methods import get_point_section_lengths
    from tmd.Tree.methods import get_point_section_branch_orders
    from tmd.Tree.methods import get_bif_term
    from tmd.Tree.methods import get_bifurcations
    from tmd.Tree.methods import get_multifurcations
    from tmd.Tree.methods import get_terminations

    def __init__(self, x, y, z, d, t, p):
        '''Constructor of tmd Tree Object

        Parameters
        ----------
        x : numpy array
            The x-coordinates of neuron's tree segments.
        y : numpy array
            The y-coordinates of neuron's tree segments.
        z : numpy array
            The z-coordinate of neuron's tree segments.
        d : numpy array
            The diameters of neuron's tree segments.
        t : numpy array
            The types (basal, apical, axon) of neuron's tree segments.
        p : numpy array
            The index of the parent of neuron's tree segments.

        Returns
        -------
        tree : Tree
            tmd Tree object
        '''
        self.x = np.array(x, dtype=float)
        self.y = np.array(y, dtype=float)
        self.z = np.array(z, dtype=float)
        self.d = np.array(d, dtype=float)
        self.t = np.array(t, dtype=int)
        self.p = np.array(p, dtype=int)
        self.dA = sp.csr_matrix((np.ones(len(self.x) - 1),
                                 (range(1, len(self.x)), self.p[1:])),
                                shape=(len(self.x), len(self.x)))

    def copy_tree(self):
        """
        Returns a deep copy of the Tree.
        """
        return copy.deepcopy(self)

    def is_equal(self, tree):
        '''Tests if all tree lists are the same'''
        eq = np.alltrue([np.allclose(self.x, tree.x, atol=1e-4),
                         np.allclose(self.y, tree.y, atol=1e-4),
                         np.allclose(self.z, tree.z, atol=1e-4),
                         np.allclose(self.d, tree.d, atol=1e-4),
                         np.allclose(self.t, tree.t, atol=1e-4),
                         np.allclose(self.p, tree.p, atol=1e-4)])
        return eq

    def rotate_xy(self, angle):
        """Rotates the tree in the x-y plane
        by the defined angle.
        """
        new_x = self.x * np.cos(angle) - self.y * np.sin(angle)
        new_y = self.x * np.sin(angle) + self.y * np.cos(angle)

        self.x = new_x
        self.y = new_y

    def move_to_point(self, point=(0, 0, 0)):
        """Moves the tree in the x-y-z plane
        so that it starts from the selected point.
        """
        self.x = self.x - (self.x[0]) + point[0]
        self.y = self.y - (self.y[0]) + point[1]
        self.z = self.z - (self.z[0]) + point[2]

    def extract_simplified(self):
        """Returns a simplified tree that corresponds
           to the start - end of the sections points
        """
        beg0, end0 = self.get_sections_2()
        sections = np.transpose([beg0, end0])

        x = np.zeros([len(sections) + 1])
        y = np.zeros([len(sections) + 1])
        z = np.zeros([len(sections) + 1])
        d = np.zeros([len(sections) + 1])
        t = np.zeros([len(sections) + 1])
        p = np.zeros([len(sections) + 1])

        x[0] = self.x[sections[0][0]]
        y[0] = self.y[sections[0][0]]
        z[0] = self.z[sections[0][0]]
        d[0] = self.d[sections[0][0]]
        t[0] = self.t[sections[0][0]]
        p[0] = - 1

        for i, s in enumerate(sections):
            x[i + 1] = self.x[s[1]]
            y[i + 1] = self.y[s[1]]
            z[i + 1] = self.z[s[1]]
            d[i + 1] = self.d[s[1]]
            t[i + 1] = self.t[s[1]]
            p[i + 1] = np.where(beg0 == s[0])[0][0]

        return Tree(x, y, z, d, t, p)



'''
tmd matching algorithms implementation
'''


def marriage_problem(women_preferences, men_preferences):
    '''Matches N women to M men so that max(M, N)
    are coupled to their preferred choice that is available
    See https://en.wikipedia.org/wiki/Stable_marriage_problem
    '''
    N = len(women_preferences)
    M = len(men_preferences)

    swapped = False

    if M > N:
        swap = women_preferences
        women_preferences = men_preferences
        men_preferences = swap
        N = len(women_preferences)
        M = len(men_preferences)
        swapped = True

    free_women = range(N)
    free_men = range(M)

    couples = {x: None for x in range(N)}  # woman first, then current husband

    while len(free_men) > 0:
        m = free_men.pop()
        choice = men_preferences[m].pop(0)

        if choice in free_women:
            couples[choice] = m
            free_women.remove(choice)
        else:
            current = np.where(np.array(women_preferences)[choice] == couples[choice])[0][0]
            tobe = np.where(np.array(women_preferences)[choice] == m)[0][0]
            if current < tobe:
                free_men.append(couples[choice])
                couples[choice] = m
            else:
                free_men.append(m)

    if swapped:
        return [(couples[k], k) for k in couples]

    return [(k, couples[k]) for k in couples]


def symmetric(p):
    '''Returns the symmetric point of a PD point on the diagonal
    '''
    return [(p[0] + p[1]) / 2., (p[0] + p[1]) / 2]


def matching_diagrams(p1, p2, plot=False, method='munkres', use_diag=True, new_fig=True, subplot=(111)):
    '''Returns a list of matching components
    Possible matching methods:
    - munkress
    - marriage problem
    '''
    from scipy.spatial.distance import cdist
    import munkres
    from tmd.view import common as _cm

    def plot_matching(p1, p2, indices, new_fig=True, subplot=(111)):
        '''Plots matching between p1, p2
        for the corresponding indices
        '''
        import pylab as plt
        fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot)
        for i, j in indices:
            ax.plot((p1[i][0], p2[j][0]), (p1[i][1], p2[j][1]), color='black')
            ax.scatter(p1[i][0], p1[i][1], c='r')
            ax.scatter(p2[j][0], p2[j][1], c='b')

    if use_diag:
        p1_enh = p1 + [symmetric(i) for i in p2]
        p2_enh = p2 + [symmetric(i) for i in p1]
    else:
        p1_enh = p1
        p2_enh = p2

    D = cdist(p1_enh, p2_enh)

    if method == 'munkres':
        m = munkres.Munkres()
        indices = m.compute(np.copy(D))
    elif method == 'marriage':
        first_pref = [np.argsort(k).tolist() for k in cdist(p1_enh, p2_enh)]
        second_pref = [np.argsort(k).tolist() for k in cdist(p2_enh, p1_enh)]
        indices = marriage_problem(first_pref, second_pref)

    if plot:
        plot_matching(p1_enh, p2_enh, indices, new_fig=new_fig, subplot=subplot)

    ssum = np.sum([D[i][j] for (i, j) in indices])

    return indices, ssum




'''
tmd statistical analysis on PH diagrams algorithms implementation
'''
import numpy as np
from scipy import stats


def get_bifurcations(ph):
    """
    Returns the bifurcations from the diagram
    """
    return np.array(ph)[:, 1]


def get_terminations(ph):
    """
    Returns the terminations from the diagram
    """
    return np.array(ph)[:, 0]


def get_lengths(ph):
    """
    Returns the lengths of the bars from the diagram
    """
    return np.array([np.abs(i[0] - i[1]) for i in ph])


def get_total_length(ph):
    """Calculates the total length of a barcode
       by summing the length of each bar. This should
       be equivalent to the total length of the tree
       if the barcode represents path distances.
    """
    return sum([np.abs(p[1] - p[0]) for p in ph])


def transform_ph_to_length(ph, keep_side='end'):
    '''Transforms a persistence diagram into a
    (start_point, length) equivalent diagram or a
    (end, length) diagram depending on keep_side option.
    Note: the direction of the diagram will be lost!
    '''
    if keep_side == 'start':
        # keeps the start point and the length of the bar
        return [[min(i), np.abs(i[1] - i[0])] for i in ph]
    else:
        # keeps the end point and the length of the bar
        return [[max(i), np.abs(i[1] - i[0])] for i in ph]


def transform_ph_from_length(ph, keep_side='end'):
    '''Transforms a persistence diagram into a
    (start_point, length) equivalent diagram or a
    (end, length) diagram depending on keep_side option.
    Note: the direction of the diagram will be lost!
    '''
    if keep_side == 'start':
        # keeps the start point and the length of the bar
        return [[i[0], i[1] - i[0]] for i in ph]
    else:
        # keeps the end point and the length of the bar
        return [[i[0] - i[1], i[0]] for i in ph]


def nosify(var, noise=0.1):
    '''Adds noise to an instance of data
    Can be used with a ph as follows:
    noisy_pd = [add_noise(d, 1.0) if d[0] != 0.0
                else [d[0],add_noise([d[1]],1.0)[0]] for d in pd]

    To output the new pd:
    F = open(...)
    for d in noisy_pd:
        towrite = '%f, %f\n'%(d[0],d[1])
        F.write(towrite)
    F.close()
    '''
    var_new = np.zeros(len(var))
    for i, v in enumerate(var):
        var_new[i] = stats.norm.rvs(v, noise)
    return var_new



# Example script to compute the diversity index from a set of classes.


def diversity_index(perc, simil, q):
    """Computes the generalized diversity index
       as described in http://onlinelibrary.wiley.com/doi/10.1890/10-2402.1/abstract
       Inputs:
            perc: list of percentages of species distribution
            of size S.
            simil: confusion matrix indicating the similarity
            between species of size SxS.
            q: the order of diversity index.
    """
    import numpy as np

    perc = np.array(perc, dtype=float) / sum(perc)

    diq = 0.0

    for i in range(len(perc)):
        zpi = np.dot(simil[i], perc)
        diq = diq + np.power(perc[i], q) * np.power(zpi, q - 1.)

    return np.power(diq, 1. / (1. - q))


def diversity_index_inf(perc, simil):
    """Computes the generalized diversity index
       as described in http://onlinelibrary.wiley.com/doi/10.1890/10-2402.1/abstract
       Inputs:
            perc: list of percentages of species distribution
            of size S.
            simil: confusion matrix indicating the similarity
            between species of size SxS.
            q: the order of diversity index is set to inf
    """
    import numpy as np

    diq = np.inf

    perc = np.array(perc, dtype=float) / sum(perc)

    for i in range(len(perc)):
        zpi = np.dot(simil[i], perc)
        diq = min(diq, zpi)

    return np.float(1.) / diq


def diversity_index_one(perc, simil):
    """Computes the generalized diversity index
       as described in http://onlinelibrary.wiley.com/doi/10.1890/10-2402.1/abstract
       Inputs:
            perc: list of percentages of species distribution
            of size S.
            simil: confusion matrix indicating the similarity
            between species of size SxS.
            q: the order of diversity index is set to one
    """
    import numpy as np

    diq = 1.0

    perc = np.array(perc, dtype=float) / sum(perc)

    for i in range(len(perc)):
        zpi = np.dot(simil[i], perc)
        diq = diq * np.power(zpi, perc[i])

    return np.float(1.) / diq


def diversity_index_zero(perc, simil):
    """Computes the generalized diversity index
       as described in http://onlinelibrary.wiley.com/doi/10.1890/10-2402.1/abstract
       Inputs:
            perc: list of percentages of species distribution
            of size S.
            simil: confusion matrix indicating the similarity
            between species of size SxS.
            q: the order of diversity index is set to one
    """
    import numpy as np

    diq = 1.0

    perc = np.array(perc, dtype=float) / sum(perc)

    for i in range(len(perc)):
        zpi = np.dot(simil[i], perc)
        diq = diq * np.power(zpi, perc[i])

    return np.float(1.) / diq

    perc = np.array(perc, dtype=float) / sum(perc)

    diq = 0.0

    for i in range(len(perc)):
        zpi = np.dot(simil[i], perc)
        if not np.close(zpi, 0.0):
            diq = diq + perc[i] / zpi

    return diq


def diversity_vary_q(perc, simil, dep=np.linspace(0.05, 0.95, 20).tolist() + range(2, 10)):
    """Computes the diversity index
       with different q values:
       from q=0 to q=infinity
    """
    # div_index = [diversity_index_zero(perc, simil), diversity_index_one(perc, simil)]

    div_index = [diversity_index(perc, simil, q) for q in dep]

    # div_index = div_index + [diversity_index_inf(perc, simil)]

    return dep, div_index



import tmd
import numpy as np
# import view

list_of_modules = ['discriminant_analysis', 'discriminant_analysis', 'tree']

list_of_classifiers =['LinearDiscriminantAnalysis', 'QuadraticDiscriminantAnalysis', 'DecisionTreeClassifier']

def train(mod, classifier, data, labels, **kwargs):
    '''Trains the classifier from mod of sklearn
       with data and targets.
       Returns a fited classifier.
    '''
    import importlib

    clas_mod = importlib.import_module('sklearn.' + mod)
    clf = getattr(clas_mod, classifier)()
    clf.set_params(**kwargs)

    clf.fit(data, labels)

    return clf

def predict(clf, data):
    '''Predict label for data for the trained classifier clf.
       Returns the index of the predicted class
       for each datapoint in data.
    '''
    predict_label = clf.predict([data])

    return predict_label[0]

def classify_cell_in_groups(list_of_groups=['L5_UTPC', 'L5_STPC', 'L5_TTPC1', 'L5_TTPC2'],
                            cell_to_classify='./L5_TTPC1/C030796A-P3.h5',
                            neurite_type='apical',
                            classifier_module=list_of_modules[0],
                            classifier_method=list_of_classifiers[0],
                            number_of_trials=20):

    # ------------------------ Training dataset --------------------------------
    # Load all data from selected folders
    groups = [tmd.io.load_population(l) for l in list_of_groups]
    # Define labels depending on the number of neurons in each folder
    labels = [i+1 for i,k in enumerate(groups) for j in k.neurons]
    # Generate a persistence diagram per neuron
    pers_diagrams = [tmd.methods.get_ph_neuron(j, neurite_type=neurite_type)
                     for i,k in enumerate(groups) for j in k.neurons]
    # Define x-ylimits
    xlims, ylims = tmd.analysis.define_limits(pers_diagrams)
    # Generate a persistence image for each diagram
    pers_images = [tmd.analysis.persistence_image_data(p, xlims=xlims, ylims=ylims)
                   for p in pers_diagrams]
    # Create the train dataset from the flatten images
    train_dataset = [i.flatten() for i in pers_images]

    # ------------------------ Test cell dataset -------------------------------
    # Load cell to be classified
    neuron2test = tmd.io.load_neuron(cell_to_classify)
    # Get persistence diagram from test cell
    pers2test = tmd.methods.get_ph_neuron(neuron2test, neurite_type=neurite_type)
    # Get persistence image from test cell
    pers_image2test = tmd.analysis.persistence_image_data(pers2test, xlims=xlims, ylims=ylims)
    # Create the test dataset from the flatten image of the test cell
    test_dataset = pers_image2test.flatten()

    predict_labels = []
    # Train classifier with training images for selected number_of_trials
    for i in xrange(number_of_trials):

        clf = train(classifier_module, classifier_method, train_dataset, labels)
        # Test classifier with test image and return predictions
        predict_labels.append(predict(clf, test_dataset))

    percentages = {groups[i-1].name: np.float(len(np.where(np.array(predict_labels) == i)[0])) / len(predict_labels)
                   for i in np.unique(labels)}

    return percentages



'''
Python module that contains the functions
about reading swc files.
'''
import re
import numpy as np


# Definition of swc data container
SWC_DCT = {'index': 0,
           'type': 1,
           'x': 2,
           'y': 3,
           'z': 4,
           'radius': 5,
           'parent': 6}


def read_swc(input_file, line_delimiter='\n'):
    '''Function to properly load a swc file
       that contains a list of sections,
       into a 'Data' format that contains all
       info extracting comments.
    '''
    # Read all data from file.
    with open(input_file, 'r') as f:

        read_data = f.read()

    # Split data per lines
    split_data = read_data.split(line_delimiter)

    # Clean data from comments and empty lines
    split_data = [a for a in split_data if '#' not in a]
    split_data = [a for a in split_data if a != '']

    return np.array(split_data)


def swc_to_data(data_swc):
    '''Transform swc to data to be used in make_tree.
    '''
    expected_data = re.compile(r'^\s*([-+]?\d*\.\d+|[-+]?\d+)'
                               r'\s*([-+]?\d*\.\d+|[-+]?\d+)\s'
                               r'*([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*$')

    data = []

    for dpoint in data_swc:

        if expected_data.match(dpoint.replace('\r', '')):

            segment_point = np.array(expected_data.match(dpoint.replace('\r', '')).groups(),
                                     dtype=np.float)

            # make the radius diameter
            segment_point[SWC_DCT['radius']] = 2. * segment_point[SWC_DCT['radius']]

            data.append(segment_point)

    return np.array(data)


def swc_data_to_lists(data):
    """
    Transforms data as loaded from read_swc
    into a set of 'meaningful' lists:

    x: list of floats
        x-coordinates

    y: list of floats
        y-coordinates

    z: list of floats
        z-coordinates

    d: list of floats
        diameters

    t: list of ints
        tree type

    p: list of ints
        parent id

    ch: dictionary
        children id(s)
    """
    length = len(data)

    # Here we define the expected structure of the data.
    # If this structure is not followed, the data will fail
    # to load and the method will be terminated, with an error message.

    expected_data = re.compile(r'^\s*([-+]?\d*\.\d+|[-+]?\d+)'
                               r'\s*([-+]?\d*\.\d+|[-+]?\d+)\s'
                               r'*([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*'
                               r'([-+]?\d*\.\d+|[-+]?\d+)\s*$')

    # Definition of swc data from SWC_DCT function

    x = np.zeros(length, dtype=float)
    y = np.zeros(length, dtype=float)
    z = np.zeros(length, dtype=float)
    d = np.zeros(length, dtype=float)
    t = np.zeros(length, dtype=int)
    p = np.zeros(length, dtype=int)
    ch = {}

    first_line_data = expected_data.match(data[0].replace('\r', ''))

    total_offset = int(first_line_data.groups()[0])

    for enline in range(length):

        segment_point = expected_data.match(data[enline].replace('\r', '')).groups()

        x[enline] = float(segment_point[SWC_DCT['x']])
        y[enline] = float(segment_point[SWC_DCT['y']])
        z[enline] = float(segment_point[SWC_DCT['z']])
        # swc contains radii, and here it is transformed into diameter.
        d[enline] = 2 * float(segment_point[SWC_DCT['radius']])
        t[enline] = int(segment_point[SWC_DCT['type']])
        if enline != 0:
            p[enline] = int(segment_point[SWC_DCT['parent']]) - total_offset
        else:
            p[enline] = int(segment_point[SWC_DCT['parent']])

        if int(segment_point[SWC_DCT['index']]) - enline != total_offset:
            raise Exception("Aborting process, with non-sequential ids error.\
                             Fix to proceed.")

    for enline in range(length):

        ch[enline] = list(np.where(p == enline)[0])

    return x, y, z, d, t, p, ch



'''
tmd class : Soma
'''
import copy
import numpy as np


class Soma(object):
    '''Class of neuron soma
    '''
    # pylint: disable=import-outside-toplevel
    from tmd.Soma.methods import get_center
    from tmd.Soma.methods import get_diameter

    def __init__(self, x=np.array([]), y=np.array([]), z=np.array([]),
                 d=np.array([])):
        """
        Constructor for tmd Soma Object

        Parameters
        ----------
        x : numpy array
            The x-coordinates of surface trace of neuron soma.
        y : numpy array
            The y-coordinates of surface trace of neuron soma.
        z : numpy array
            The z-coordinate of surface trace of neuron soma.
        d : numpy array
            The diameters of surface trace of neuron soma.
        ----------
        """
        self.x = np.array(x, dtype=float)
        self.y = np.array(y, dtype=float)
        self.z = np.array(z, dtype=float)
        self.d = np.array(d, dtype=float)

    def copy_soma(self):
        """
        Returns a deep copy of the Soma.
        """
        return copy.deepcopy(self)

    def is_equal(self, soma):
        '''Tests if all soma data are the same'''
        eq = np.alltrue([np.allclose(self.x, soma.x, atol=1e-4),
                         np.allclose(self.y, soma.y, atol=1e-4),
                         np.allclose(self.z, soma.z, atol=1e-4),
                         np.allclose(self.d, soma.d, atol=1e-4)])
        return eq



"""
    TopologyNeuron

    Author: Lida Kanari

    Ecole Polytechnique Federale de Lausanne
    Blue Brain Project

    (c) 2014-2015. All rights reserved.
"""


from tmd.io import io
from tmd.Soma import Soma
from tmd.Tree import Tree
from tmd.Neuron import Neuron
from tmd.Population import Population
from tmd.Topology import methods
from tmd.Topology import analysis
from tmd.Topology import statistics
from tmd import utils




def find_matching(p1, p2):
    '''Finds the best matching between two
    persistent diagrams and returns a list
    of matched indices. The difference is
    computed between p2 from p1, and
    the matching starts from the longest
    components.    
    '''
    from scipy.spatial import distance
    from view import plot

    p2 = sort_ph(p2)
    p2_symm = [((p[0] + p[1]) / 2., (p[0] + p[1]) / 2.) for p in p2]
    avail = range(len(p1))
    avail_symm = range(len(p2_symm))

    plot.ph_diagram(p1)
    plot.ph_diagram(p2, new_fig=False, color='r')

    for p in p2:
        if avail:
            index = np.argmin( [np.linalg.norm(np.subtract(p[:2], pi)) for pi in np.array(p1)[avail]])
            value = np.array(p1)[avail][index]
            print p[:2], value, np.where(p1 == value)[0][0]
            plt.plot([p[:2][0], value[0]], [p[:2][1], value[1]], c='b')
            avail.remove(np.where(p1 == value)[0][0])
        else:
            index = np.argmin( [np.linalg.norm(np.subtract(p[:2], pi)) for pi in np.array(p2_symm)[avail_symm]])
            value = np.array(p2_symm)[avail_symm][index]
            print p[:2], value, np.where(p2_symm == value)[0][0]
            plt.plot([p[:2][0], value[0]], [p[:2][1], value[1]], c='b')
            avail_symm.remove(np.where(p2_symm == value)[0][0])

    
def find_matching_time_series(p_list):
    '''Finds the best matching between two
    persistent diagrams and returns a list
    of matched indices. The difference is
    computed between p2 from p1, and
    the matching starts from the longest
    components.    
    '''
    from scipy.spatial import distance
    from view import plot

    p2 = sort_ph(p2)
    p1_symm = [((p[0] + p[1]) / 2., (p[0] + p[1]) / 2.) for p in p1]
    avail = range(len(p1))
    avail_symm = range(len(p1_symm))

    plot.ph_diagram(p1)
    plot.ph_diagram(p2, new_fig=False, color='r')

    for p in p2:
        if avail:
            index = np.argmin( [np.linalg.norm(np.subtract(p[:2], pi)) for pi in np.array(p1)[avail]])
            value = np.array(p1)[avail][index]
            print p[:2], value, np.where(p1 == value)[0][0]
            plt.plot([p[:2][0], value[0]], [p[:2][1], value[1]], c='b')
            avail.remove(np.where(p1 == value)[0][0])
        else:
            index = np.argmin( [np.linalg.norm(np.subtract(p[:2], pi)) for pi in np.array(p1_symm)[avail_symm]])
            value = np.array(p1_symm)[avail_symm][index]
            print p[:2], value, np.where(p1_symm == value)[0][0]
            plt.plot([p[:2][0], value[0]], [p[:2][1], value[1]], c='b')
            avail_symm.remove(np.where(p1_symm == value)[0][0])


def marriage_problem(women_preferences, men_preferences):
    '''Matches N women to M men so that max(M, N)
    are coupled to their preferred choice that is available
    See https://en.wikipedia.org/wiki/Stable_marriage_problem
    '''
    N = len(women_preferences)
    M = len(men_preferences)

    swapped = False

    if M > N:
        swap = women_preferences
        women_preferences = men_preferences
        men_preferences = swap
        N = len(women_preferences)
        M = len(men_preferences)
        swapped = True

    free_women = range(N)
    free_men = range(M)

    couples = {x: None for x in xrange(N)} # woman first, then current husband

    count = 0

    while len(free_men) > 0:
        m = free_men.pop()
        choice = men_preferences[m].pop(0)

        if choice in free_women:
            couples[choice] = m
            free_women.remove(choice)
        else:
            current = np.where(np.array(women_preferences)[choice] == couples[choice])[0][0]
            tobe = np.where(np.array(women_preferences)[choice] == m)[0][0]
            if  current < tobe:
                free_men.append(couples[choice])
                couples[choice] = m
            else:
                free_men.append(m)

    if swapped:
        return {couples[k]: k for k in couples}

    return couples


def marry_components(p1, p2, ax=None, z1=0, z2=0.2):
    '''Returns a list of matching components
    '''
    from scipy.spatial.distance import cdist
    from mpl_toolkits.mplot3d.art3d import Line3DCollection

    #if ax is None:
        #fig = plt.figure()
        #ax = fig.add_subplot(111, projection='3d')

    first_distances = cdist(p1, p2)
    second_distances = cdist(p2, p1)

    first_pref = [np.argsort(k).tolist() for k in first_distances]
    second_pref = [np.argsort(k).tolist() for k in second_distances]

    church = marriage_problem(first_pref, second_pref)

    #ax.scatter3D(np.transpose(p1)[0], np.transpose(p1)[1], z1)
    #ax.scatter3D(np.transpose(p2)[0], np.transpose(p2)[1], z2, c='r')
    #ax.plot3D([0,200], [0,200], z1, c='g')
    #ax.plot3D([0,200], [0,200], z2, c='g')

    def symmetric(i, j):
        return ((i + j) / 2., (i + j) / 2.)

    speed = []

    for c in church:
        if c is None:
            x, y = symmetric(p2[church[c]][0], p2[church[c]][1])
            #ax.plot3D([x, p2[church[c]][0]], [y, p2[church[c]][1]], [z1,z2], c='b')
            sp = np.linalg.norm(np.subtract([x, p2[church[c]][0]], [y, p2[church[c]][1]]))
        elif church[c] is not None:
            #ax.plot3D([p1[c][0], p2[church[c]][0]], [p1[c][1], p2[church[c]][1]], [z1,z2], c='b')
            sp = np.linalg.norm(np.subtract([p1[c][0], p2[church[c]][0]], [p1[c][1], p2[church[c]][1]]))
        else:
            x, y = symmetric(p1[c][0], p1[c][1])
            #ax.plot3D([p1[c][0], x], [p1[c][1], y], [z1,z2], c='b')
            sp = np.linalg.norm(np.subtract([p1[c][0], x], [p1[c][1], y]))

        speed.append(sp)

    return speed

def vineyards(ph_list):
    from matplotlib import pylab as plt
    from mpl_toolkits.mplot3d import Axes3D

    #fig = plt.figure()
    #ax = fig.add_subplot(111, projection='3d')

    speeds = []

    for ip, p in enumerate(ph_list[:-1]):
        sp = marry_components(ph_list[ip], ph_list[ip+1], z1=ip*0.2, z2=(ip + 1)*0.2)
        speeds.append(sp)
        #ax = plt.gca()

    return speeds


def get_persistence_diagram_timelapse(trees, **kwargs):
    '''Method to extract ph from tree that contains mutlifurcations'''
    ph = []

    for itr, tree in enumerate(trees):
        rd = getattr(tree, 'get_point_radial_distances_time')(time=itr, **kwargs)

        active = tree.get_bif_term() == 0

        beg, end = tree.get_sections_3()

        beg = np.array(beg)
        end = np.array(end)

        parents = {e: b for b, e in zip(beg, end)}
        children = {b: end[np.where(beg == b)[0]] for b in np.unique(beg)}

        while len(np.where(active)[0]) > 1:
            alive = list(np.where(active)[0])
            for l in alive:

                p = parents[l]
                c = children[p]

                if np.alltrue(active[c]):
                    active[p] = True
                    active[c] = False

                    mx = np.argmax(abs(rd[c]))
                    mx_id = c[mx]

                    alive.remove(mx_id)
                    c = np.delete(c, mx)

                    for ci in c:
                        ph.append([rd[ci], rd[p]])
                        alive.remove(ci)

                    alive.append(p)

                    rd[p] = rd[mx_id]

        ph.append([rd[np.where(active)[0][0]], 0]) # Add the last alive component

    return ph



'''
Python module that contains the functions
about reading h5 files.
'''

import numpy as np
import h5py
from tmd.io.swc import SWC_DCT

# Definition of h5 data container
# separated in points (PX,PY,PZ,PD)
# and groups (GPFIRST, GTYPE, GPID)
h5_dct = {'PX': 0,
          'PY': 1,
          'PZ': 2,
          'PD': 3,
          'GPFIRST': 0,
          'GTYPE': 1,
          'GPID': 2}


def _find_group(point_id, groups):
    '''Find the structure group a points id belongs to.
       Return: group or section point_id belongs to. Last group if
               point_id out of bounds.
    '''
    bs = np.searchsorted(groups[:, h5_dct['GPFIRST']], point_id, side='right')
    bs = max(bs - 1, 0)
    return groups[bs]


def _find_parent_id(point_id, groups):
    '''Find the parent ID of a point'''
    group = _find_group(point_id, groups)
    if point_id != group[h5_dct['GPFIRST']]:
        # point is not first point in section
        # so parent is previous point
        return point_id - 1
    # parent is last point in parent group
    parent_group_id = group[h5_dct['GPID']]
    # get last point in parent group
    return groups[parent_group_id + 1][h5_dct['GPFIRST']] - 1


def _find_last_point(group_id, groups, points):
    ''' Identifies and returns the id of the last point of a group'''
    group_initial_ids = np.sort(np.transpose(groups)[0])

    if group_id != len(group_initial_ids) - 1:
        return group_initial_ids[np.where(group_initial_ids == groups[group_id][0])[0][0] + 1] - 1
    return len(points) - 1


def remove_duplicate_points(points, groups):
    ''' Removes the duplicate points from the beginning of a section,
    if they are present in points-groups representation.
    Returns:
       points, groups with unique points.
    '''

    group_initial_ids = groups[:, h5_dct['GPFIRST']]

    to_be_reduced = np.zeros(len(group_initial_ids))
    to_be_removed = []

    for ig, g in enumerate(groups):
        iid, typ, pid = g[h5_dct['GPFIRST']], g[h5_dct['GTYPE']], g[h5_dct['GPID']]
        # Remove first point from sections that are
        # not the root section, a soma, or a child of a soma
        if pid != -1 and typ != 1 and groups[pid][h5_dct['GTYPE']] != 1:
            # Remove duplicate from list of points
            to_be_removed.append(iid)
            # Reduce the id of the following sections
            # in groups structure by one
            to_be_reduced[ig + 1:] += 1

    groups[:, h5_dct['GPFIRST']] = groups[:, h5_dct['GPFIRST']] - to_be_reduced
    points = np.delete(points, to_be_removed, axis=0)

    return points, groups


def _unpack_data(points, groups):
    '''Unpack data from h5 data groups into internal format'''
    return np.array([(i, _find_group(i, groups)[h5_dct['GTYPE']],
                     p[h5_dct['PX']], p[h5_dct['PY']],
                     p[h5_dct['PZ']], p[h5_dct['PD']],
                     _find_parent_id(i, groups))
                     for i, p in enumerate(points)])


def _unpack_v1(h5file):
    '''Unpacks data of h5_v1 file
       in a simplified data structure.
    '''
    points = np.array(h5file['points'])
    groups = np.array(h5file['structure'])
    return points, groups


def _unpack_v2(h5file, stage):
    '''Unpack groups from HDF5 v2 file'''
    if stage == 'unraveled':
        stage1 = 'raw'
    else:
        stage1 = stage
    points = np.array(h5file['neuron1/%s/points' % stage])
    groups = np.array(h5file['neuron1/structure/%s' % stage1])
    stypes = np.array(h5file['neuron1/structure/sectiontype'])
    groups = np.hstack([groups, stypes])
    groups[:, [1, 2]] = groups[:, [2, 1]]
    return points, groups


def _get_h5_version(h5file):
    '''Determine whether an HDF5 file is v1 or v2.

    Returns:
        1, 2 or None
    '''
    if 'points' in h5file and 'structure' in h5file:
        return 1
    if 'neuron1/structure' in h5file:
        return 2
    return None


def read_h5(input_file, remove_duplicates=True):
    '''Function to properly load sn h5 file,
       of v1 or v2 format.
    '''
    h5file = h5py.File(input_file, mode='r')
    version = _get_h5_version(h5file)

    if version == 1:
        points, groups = _unpack_v1(h5py.File(input_file, mode='r'))
    elif version == 2:
        stg = next(s for s in ('repaired', 'unraveled', 'raw', '0', '1', '2')
                   if s in h5file['neuron1'])
        points, groups = _unpack_v2(h5py.File(input_file, mode='r'),
                                    stage=stg)
    else:
        raise Exception("Not recognized h5 version")

    h5file.close()

    if remove_duplicates:
        return _unpack_data(*remove_duplicate_points(points, groups))
    return _unpack_data(points, groups)


def h5_data_to_lists(data):
    """
    Transforms data as loaded from read_h5
    into a set of 'meaningful' lists:

    x: list of floats
        x-coordinates

    y: list of floats
        y-coordinates

    z: list of floats
        z-coordinates

    d: list of floats
        diameters

    t: list of ints
        tree type

    p: list of ints
        parent id

    ch: dictionary
        children id(s)

    """
    length = len(data)

    x = np.transpose(data)[SWC_DCT['x']]
    y = np.transpose(data)[SWC_DCT['y']]
    z = np.transpose(data)[SWC_DCT['z']]
    d = np.transpose(data)[SWC_DCT['radius']]
    t = np.transpose(data)[SWC_DCT['type']]
    p = np.transpose(data)[SWC_DCT['parent']]
    ch = {}

    for enline in range(length):

        ch[enline] = list(np.where(p == enline)[0])

    return x, y, z, d, t, p, ch



""" Distribution configuration for TMD
"""
from setuptools import setup
from setuptools import find_packages
import imp


config = {
    'description': 'TMD: a python package for the topological analysis of neurons',
    'author': 'Lida Kanari',
    'url': 'https://github.com/BlueBrain/TMD',
    'author_email': 'lida.kanari@epfl.ch',
    'install_requires': [
        'h5py>=2.8.0',
        'enum-compat>=0.0.1',
        'scipy>=0.13.3',
        'numpy>=1.8.0',
        'scikit-learn>=0.19.1',
        'munkres>=1.0.12',
    ],
    'setup_requires':['setuptools_scm'],
    'extras_require': {
                       'viewer': ['matplotlib>=1.3.1',],
                       },
    'packages': find_packages(),
    'license': 'LGPL',
    'scripts': [],
    'name': 'tmd',
    'include_package_data': True,
    'use_scm_version': True,
}

setup(**config)



def write_swc_from_data(output_file, data, line_delimiter='\n'):
    '''Write swc'''
    ffile = open(output_file, 'w')

    for line in data:
        ffile.write(line + line_delimiter)

    ffile.close()



"""
    tmd : Neuron class

    Author: Lida Kanari

    Ecole Polytechnique Federale de Lausanne
    Blue Brain Project

    (c) 2014-2015. All rights reserved.
"""

from tmd.Population import Population



import tmd
from view import polar_plots

filename = './'

pop = tmd.io.load_population(filename)

# Get the data for a single cell
res = polar_plots.get_histogram_polar_coordinates(pop.neurons[0], neurite_type='basal', N=30)

# Plot the data for a single cell
polar_plots.plot_polar_coordinates(res)

# Extract the polar plots of all neurons in the population and save images
# in a selected directory
for n in pop.neurons:
    res = polar_plots.get_histogram_polar_coordinates(n, neurite_type='basal', N=30)
    polar_plots.plot_polar_coordinates(res, output_path='./PolarPlots/', output_name=n.name.split('/')[-1])
    plt.close()
